searchData={"items":[{"type":"module","title":"ExDTLS","doc":"Module that allows performing DTLS handshake including a DTLS-SRTP one.\n\n`ExDTLS` executes native OpenSSL functions to perform DTLS handshake.\nIt doesn't create or require any socket.\nInstead, it returns generated DTLS packets, which then have to be transported to the peer.","ref":"ExDTLS.html"},{"type":"function","title":"ExDTLS.do_handshake/1","doc":"Starts performing DTLS handshake.\n\nGenerates initial DTLS packets that have to be passed to the second host.\nHas to be called by a host working in the client mode.\n\n`timeout` is a time in ms after which `handle_timeout/1` should be called.","ref":"ExDTLS.html#do_handshake/1"},{"type":"function","title":"ExDTLS.generate_key_cert/2","doc":"Generates a new key/certificate pair.\n\nThis is always 2048-bit RSA key.\n\n`not_before` and `not_after` can be used to\nspecify certificate duration in seconds.\nThey have to fit into architecture-dependent integer size.\nDefaults to (-1 year) - (+ 1 year).\n\nReturns DER representation in binary format.","ref":"ExDTLS.html#generate_key_cert/2"},{"type":"function","title":"ExDTLS.get_cert/1","doc":"Gets current, local certificate.\n\nReturns DER representation in binary format.","ref":"ExDTLS.html#get_cert/1"},{"type":"function","title":"ExDTLS.get_cert_fingerprint/1","doc":"Returns an SHA-256 digest of the DER representation of the X509 certificate.","ref":"ExDTLS.html#get_cert_fingerprint/1"},{"type":"function","title":"ExDTLS.get_peer_cert/1","doc":"Gets peer certificate.\n\nReturns DER representation in binary format or `nil`\nwhen no certificate was presented by the peer or no connection\nwas established.","ref":"ExDTLS.html#get_peer_cert/1"},{"type":"function","title":"ExDTLS.get_pkey/1","doc":"Gets current, local private key.\n\nReturns key specific representation in binary format.","ref":"ExDTLS.html#get_pkey/1"},{"type":"function","title":"ExDTLS.handle_data/2","doc":"Handles peer's packets.\n\nWhen handshake is finished, calling `handle_data` will return `{:ok, binary()}`,\nwhich is decoded data.\n\n`:handshake_packets` contains handshake data that has to be sent to the peer.\n`:handshake_want_read` means some additional data is needed for continuing handshake.\nIt can be returned when retransmitted packet was passed but timer didn't expired yet.\n`timeout` is a time in ms after which `handle_timeout/1` should be called.\n\nBoth local and remote keying materials consist of `master key` and `master salt`.","ref":"ExDTLS.html#handle_data/2"},{"type":"function","title":"ExDTLS.handle_timeout/1","doc":"Handles timeout.\n\nIf there is a timeout to handle, this function will return `packets` that has\nto be retransmitted and a new timeout in ms after which `handle_timeout/1` should\nbe called once agian.\n\nIf there is no timeout to handle, simple `{:ok, dtls()}` tuple is returned.","ref":"ExDTLS.html#handle_timeout/1"},{"type":"function","title":"ExDTLS.init/1","doc":"Initializes `ExDTLS`.\n\nAccepts a keyword list with the following options (`t:opts_t/0`):\n* `mode` - `:client` if ExDTLS module should work as a client or `:server` if as a server.\n  This option is required.\n* `dtls_srtp` - `true` if DTLS-SRTP handshake should be performed or `false` if a normal one.\n  Defaults to `false`.\n* `pkey` - private key to use in this SSL context. Must correspond to `cert`.\n  If both `pkey` and `cert` are not passed, `ExDTLS` will generate 2048-bit RSA key and certificate on its own.\n* `cert` - certificate to use in this SSL context. Must correspond to `pkey`.\n  If both `pkey` and `cert` are not passed, `ExDTLS` will generate 2048-bit RSA key and certificate on its own.\n* `verify_peer` - `true` if peer's certificate should be verified.\n  Default OpenSSL verification is performed except that:\n    * self-signed certificates are also accepted\n    * verification fails if there is no peer cert\n\n  Under the hood, `ExDTLS` uses `SSL_CTX_set_verify` with `SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT`.\n  Note that if `verify_peer` is `false`, `get_peer_cert/1` called on `ExDTLS` working in the\n  server mode, will always return `nil`. Defaults to `false`.","ref":"ExDTLS.html#init/1"},{"type":"function","title":"ExDTLS.write_data/2","doc":"Writes data to the DTLS connection.\n\nGenerates encrypted packets that need to be passed to the second host.","ref":"ExDTLS.html#write_data/2"},{"type":"type","title":"ExDTLS.dtls/0","doc":"A reference to `ExDTLS` native.","ref":"ExDTLS.html#t:dtls/0"},{"type":"type","title":"ExDTLS.opts_t/0","doc":"Type describing ExDTLS configuration.\n\nSee `init/1` for the meaning of each option","ref":"ExDTLS.html#t:opts_t/0"},{"type":"type","title":"ExDTLS.protection_profile_t/0","doc":"Supported protection profiles.\n\nFor meaning of these values please refer to\nhttps://www.iana.org/assignments/srtp-protection/srtp-protection.xhtml","ref":"ExDTLS.html#t:protection_profile_t/0"},{"type":"extras","title":"ExDTLS","doc":"# ExDTLS\n\n[![Hex.pm](https://img.shields.io/hexpm/v/ex_dtls.svg)](https://hex.pm/packages/ex_dtls)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/ex_dtls/)\n[![CI](https://img.shields.io/github/actions/workflow/status/elixir-webrtc/ex_dtls/ci.yml?logo=github&label=CI)](https://github.com/elixir-webrtc/ex_dtls/actions/workflows/ci.yml)\n[![codecov](https://codecov.io/gh/elixir-webrtc/ex_dtls/graph/badge.svg?token=E98NHC8B00)](https://codecov.io/gh/elixir-webrtc/ex_dtls)\n\nDTLS and DTLS-SRTP library for Elixir, based on [OpenSSL].\n\n`ExDTLS` allows a user to perform DTLS handshake (including DTLS-SRTP one)\nwithout requiring any socket.\nInstead, it generates DTLS packets that a user has to transport to the peer.\nThanks to this DTLS handshake can be performed on the third-party socket e.g. one used to\nestablish a connection via ICE protocol.\n\nStarting from v0.16.0, `ExDTLS` can also be used to send arbitrary data using DTLS datagrams, see `ExDTLS.write_data/2`.","ref":"readme.html"},{"type":"extras","title":"Installation - ExDTLS","doc":"The package can be installed by adding `ex_dtls` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_dtls, \"~> 0.16.0\"}\n  ]\nend\n```\n\nPlease note that `ex_dtls` uses OpenSSL under the hood.\nWe use `pkg-config` to search for include and lib files so you must make sure\nthat `openssl.pc` file is searchable by `pkg-config`.\nIf it is not in the default `pkg-config` search path, locate your `openssl.pc`\nand add its parent directory to the `PKG_CONFIG_PATH`.\nFor example:\n\n```sh\nexport PKG_CONFIG_PATH=$PKG_CONFIG_PATH:/opt/homebrew/Cellar/openssl@3/3.1.0/lib/pkgconfig\n```\n\nIf OpenSSL was installed using `brew`, you can use the following syntax\nto make sure that the path will still be correct after OpenSSL update:\n\n```sh\nexport PKG_CONFIG_PATH=$PKG_CONFIG_PATH:$(brew --prefix openssl@3)/lib/pkgconfig\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - ExDTLS","doc":"`ExDTLS` uses OpenSSL under the hood.\nMake sure you have it installed on your OS.\n\nInit `ExDTLS` on both peers with:\n\n```elixir\n# One peer should be a client (use `mode: :client`) and the other\n# one a server (use `mode: :server`).\n# DTLS-SRTP is the most common use case for ExDTLS, we'll enable it.\ndtls = ExDTLS.init(mode: :client, dtls_srtp: true)\n```\n\nOn a peer running in a client mode start performing DTLS handshake\n\n```elixir\n{packets, timeout} = ExDTLS.do_handshake(dtls)\n```\n\nYou will obtain initial handshake packets and a `timeout`.\n`packets` has to be passed to the second peer (using your own socket UDP).\n`timeout` is a time after which `ExDTLS.handle_timeout/1` should be called.\n\nAfter receiving initial DTLS packets on the second peer pass them to `ExDTLS`:\n\n```elixir\n{:handshake_packets, packets, timeout} = ExDTLS.handle_data(dtls, packets)\n```\n\nAs a result, we will also get some new packets that have to be passed to the first peer.\n\nAfter some back and forth DTLS handshake should be finished successfully.\nThe peer that finishes the handshake first will return `{:handshake_finished, local_keying_material, remote_keying_material, protection_profile, packets}` tuple.\nThese packets have to be sent to the second peer, so it can finish its handshake too and\nreturn `{:handshake_finished, local_keying_material, remote_keying_material, protection_profile}` tuple.\n\nFor more complete examples please refer to [ex_webrtc] where we use `ex_dtls`\nor to our integration tests.","ref":"readme.html#usage"},{"type":"extras","title":"Debugging - ExDTLS","doc":"Add `compiler_flags: [\"-DEXDTLS_DEBUG\"],` in `bundlex.exs` to\nget debug logs from the native code.","ref":"readme.html#debugging"},{"type":"extras","title":"Copyright and License - ExDTLS","doc":"Copyright 2020, [Software Mansion](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=ex_dtls)\n\n[![Software Mansion](https://logo.swmansion.com/logo?color=white&variant=desktop&width=200&tag=membrane-github)](https://swmansion.com/?utm_source=git&utm_medium=readme&utm_campaign=ex_dtls)\n\nLicensed under the [Apache License, Version 2.0](LICENSE)\n\n[OpenSSL]: https://www.openssl.org/\n[ex_webrtc]: https://github.com/elixir-webrtc/ex_webrtc","ref":"readme.html#copyright-and-license"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS\n\n   APPENDIX: How to apply the Apache License to your work.\n\n      To apply the Apache License to your work, attach the following\n      boilerplate notice, with the fields enclosed by brackets \"[]\"\n      replaced with your own identifying information. (Don't include\n      the brackets!)  The text should be enclosed in the appropriate\n      comment syntax for the file format. We also recommend that a\n      file or class name and description of purpose be included on the\n      same \"printed page\" as the copyright notice for easier\n      identification within third-party archives.\n\n   Copyright 2020 Software Mansion\n\n   Licensed under the Apache License, Version 2.0 (the \"License\");\n   you may not use this file except in compliance with the License.\n   You may obtain a copy of the License at\n\n       http://www.apache.org/licenses/LICENSE-2.0\n\n   Unless required by applicable law or agreed to in writing, software\n   distributed under the License is distributed on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n   See the License for the specific language governing permissions and\n   limitations under the License.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}